@@ -27,6 +27,11 @@ something like:     macros=MMMmacros.Macros().
 """
 
 import random
+import datetime
+import re
+import types
+from os import environ
+from os import path
 
 import MMA.midiC
 import MMA.translate
@@ -41,24 +46,29 @@ import MMA.swing
 import MMA.ornament
 import MMA.rpitch
 import MMA.chords
-
+import MMA.debug
+from MMA.safe_eval import safeEnv, safeEval
 from . import gbl
-from MMA.notelen import getNoteLen
-from MMA.keysig import keySig
-from MMA.timesig import timeSig
-from MMA.lyric import lyric
-from MMA.common import *
-from MMA.safe_eval import safe_eval
+
+from   MMA.notelen import getNoteLen
+from   MMA.keysig import keySig
+from   MMA.timesig import timeSig
+from   MMA.lyric import lyric
+from   MMA.common import *
+
 
 def sliceVariable(p, sl):
     """ Slice a variable. Used by macro expand. """
 
     try:
+        # Important: We are using the non-safe version of eval()
+        #            changing to safe_eval() will stop slice
+        #            args from working!!! So, don't change it.
         new = eval('p' + "[" + sl + "]")
     except IndexError:
         error("Index '%s' out of range." % sl)
     except:
-        error("Index error in '%s'." % sl)
+        error("Index error '%s' in '%s' Check the array being sliced." % (sl,p) )
 
     if ":" not in sl:
         new = [new]
@@ -79,8 +89,8 @@ class Macros:
         if ln:
             error("VarClear does not take an argument.")
         self.vars = {}
-        if gbl.debug:
-            print("All variable definitions cleared.")
+        if MMA.debug.debug:
+            dPrint("All variable definitions cleared.")
 
     def stackValue(self, s):
         self.pushstack.append(' '.join(s))
@@ -88,12 +98,39 @@ class Macros:
     def sysvar(self, s):
         """ Create an internal macro. """
         
+        # Check for system functions.
+        
+        m = re.match( r'([^\(]+)\((.*)\)$', s )
+        if m:
+            return self.sysfun( m.group(1), m.group(2) )
+            
         # Simple/global     system values
 
         if s == 'CHORDADJUST':
             return ' '.join([ "%s=%s" % (a, MMA.chords.cdAdjust[a]) 
                               for a in sorted(MMA.chords.cdAdjust)])
 
+        elif s == 'FILENAME':
+            a = gbl.inpath.fname
+            if isinstance(a, int):
+                return ''
+            else:
+                return str(gbl.inpath.fname)
+
+        elif s == 'FILEPATH':
+            a = gbl.inpath.fname
+            if isinstance(a, int):
+                return ''
+            else:
+                return path.abspath(gbl.inpath.fname)
+
+        elif s == 'SONGPATH':
+            a = gbl.infile
+            if isinstance(a, int):
+                return ''
+            else:
+                return path.abspath(gbl.infile)
+        
         elif s == 'KEYSIG':
             return keySig.getKeysig()
 
@@ -111,6 +148,9 @@ class Macros:
 
         elif s == 'OFFSET':
             return str(gbl.tickOffset)
+        
+        elif s == 'SONGFILENAME':
+            return str(gbl.infile)
 
         elif s == 'VOLUME':
             return str(int(MMA.volume.volume * 100))  # INT() is important
@@ -125,7 +165,7 @@ class Macros:
             return MMA.grooves.currentGroove
 
         elif s == 'GROOVELIST':
-            return ' '.join(sorted([x for x in MMA.grooves.glist.keys() if type(x) == type('')]))
+            return ' '.join(sorted([x for x in MMA.grooves.glist.keys() if isinstance(x, str)]))
 
         elif s == 'TRACKLIST':
             return ' '.join(sorted(gbl.tnames.keys()))
@@ -137,6 +177,14 @@ class Macros:
             from MMA.regplug import simplePlugs  # to avoid circular import error
             return ' '.join(simplePlugs)
 
+        elif s == 'TRACKPLUGINS':
+            from MMA.regplug import trackPlugs  # to avoid circular import error
+            return ' '.join(trackPlugs)
+         
+        elif s == 'DATAPLUGINS':
+            from MMA.regplug import dataPlugs  # to avoid circular import error
+            return ' '.join(dataPlugs)
+        
         elif s == 'SEQ':
             return str(gbl.seqCount)
 
@@ -155,7 +203,7 @@ class Macros:
 
         elif s == 'TICKPOS':
             return str(gbl.tickOffset)
-
+        
         elif s == 'TRANSPOSE':
             return str(gbl.transpose)
 
@@ -165,21 +213,11 @@ class Macros:
             return self.pushstack.pop()
 
         elif s == 'DEBUG':
-            return "Debug=%s  Filenames=%s Patterns=%s " \
-                "Sequence=%s Runtime=%s Warnings=%s Expand=%s " \
-                "Roman=%s Plectrum=%s Groove=%s" % \
-                (gbl.debug, gbl.showFilenames, gbl.pshow, gbl.seqshow,
-                 gbl.showrun,  int(not gbl.noWarn), gbl.showExpand,
-                 gbl.rmShow, gbl.plecShow, gbl.gvShow)
-
+            return MMA.debug.getFlags()
+        
         elif s == 'LASTDEBUG':
-            return "Debug=%s  Filenames=%s Patterns=%s " \
-                "Sequence=%s Runtime=%s Warnings=%s Expand=%s " \
-                "Roman=%s Plectrum=%s Groove=%s" % \
-                (gbl.Ldebug, gbl.LshowFilenames, gbl.Lpshow, gbl.Lseqshow,
-                 gbl.Lshowrun,  int(not gbl.LnoWarn), gbl.LshowExpand,
-                 gbl.LrmShow, gbl.LplecShow, gbl.LgvShow)
-
+            return MMA.debug.getLFlags()
+            
         elif s == 'VEXPAND':
             if self.expandMode:
                 return "On"
@@ -194,9 +232,6 @@ class Macros:
         elif s == "MIDISPLIT":
             return ' '.join([str(x) for x in MMA.midi.splitChannels])
 
-        elif s.startswith("NOTELEN(") and s.endswith(")"):
-            return "%sT" % getNoteLen(s[8:-1])
-            
         elif s == 'SEQRNDWEIGHT':
             return ' '.join([str(x) for x in MMA.seqrnd.seqRndWeight])
 
@@ -230,6 +265,18 @@ class Macros:
         elif s == 'LYRIC':
             return lyric.setting()
 
+        # Some time/date macros. Useful for generating copyright strings
+
+        elif s == 'DATEYEAR':
+            return str(datetime.datetime.now().year)
+
+        elif s == 'DATEDATE':
+            return datetime.datetime.now().strftime("%Y-%m-%d")
+
+        elif s == 'DATETIME':
+            return datetime.datetime.now().strftime("%H:%M:%S")
+
+        
         # Track vars ... these are in format TRACKNAME_VAR
 
         a = s.rfind('_')
@@ -320,11 +367,11 @@ class Macros:
 
         elif func == 'ORNAMENT':
             return MMA.ornament.getOrnOpts(t)
-
+        
         elif func == 'PLUGINS':
             from MMA.regplug import trackPlugs  # avoids circular import
             return ' '.join(trackPlugs)
-
+        
         elif func == 'RANGE':
             return ' '.join([str(x) for x in t.chordRange])
 
@@ -417,8 +464,14 @@ class Macros:
             return MMA.trigger.getTriggerOptions(t)
 
         elif func == 'TONE':
-            if t.vtype != "DRUM":
-                error("Only DRUM tracks have TONE")
+            if t.vtype in ('MELODY', 'SOLO'):
+                if not t.drumType:
+                    error("Melody/Solo tracks must be DRUMTYPE for tone.")
+                return str(MMA.midiC.valueToDrum(t.drumTone))
+
+            elif t.vtype != 'DRUM':
+                error("Tracktype %s doesn't have TONE" % t.vtype)
+
             return ' '.join([MMA.midiC.valueToDrum(a) for a in t.toneList])
 
         elif func == 'UNIFY':
@@ -440,6 +493,16 @@ class Macros:
         else:
             error("Unknown system track variable %s" % s)
 
+    def sysfun(self, func, arg):
+        if func == 'NOTELEN':
+            return "%sT" % getNoteLen(arg)
+        
+        elif func == 'ENV':
+            return safeEnv(arg)
+        
+        else:
+            error("Unknown system function %s" % func)
+
     def expand(self, l):
         """ Loop though input line and make variable subsitutions.
             MMA variables are pretty simple ... any word starting
@@ -569,7 +632,7 @@ class Macros:
                     if s2 >= max:
                         error("Unmatched delimiter in '%s'." % l)
 
-                l = l[:s1] + str(safe_eval(l[s1+2:s2].strip())) + l[s2+1:]
+                l = l[:s1] + str( safeEval(l[s1+2:s2].strip())) + l[s2+1:]
 
             l = l.split()
 
@@ -623,8 +686,8 @@ class Macros:
 
         self.vars[v] = random.choice(ln[1:])
 
-        if gbl.debug:
-            print("Variable $%s randomly set to '%s'" % (v, self.vars[v]))
+        if MMA.debug.debug:
+            dPrint("Variable $%s randomly set to '%s'" % (v, self.vars[v]))
 
     def newsetvar(self, ln):
         """ Set a new variable. Ignore if already set. """
@@ -638,7 +701,7 @@ class Macros:
         self.setvar(ln)
 
     def setvar(self, ln):
-        """ Set a variable. Not the difference between the next 2 lines:
+        """ Set a variable. Note the difference between the next 2 lines:
                 Set Bar BAR
                 Set Foo AAA BBB $bar
                    $Foo == "AAA BBB BAR"
@@ -667,8 +730,8 @@ class Macros:
 
         self.vars[v] = t
 
-        if gbl.debug:
-            print("Variable $%s == '%s'" % (v, self.vars[v]))
+        if MMA.debug.debug:
+            dPrint("Variable $%s == '%s'" % (v, self.vars[v]))
 
     def msetvar(self, ln):
         """ Set a variable to a number of lines. """
@@ -706,8 +769,8 @@ class Macros:
         if v in self.vars:
             del(macros.vars[v])
 
-            if gbl.debug:
-                print("Variable '%s' UNSET" % v)
+            if MMA.debug.debug:
+                dPrint("Variable '%s' UNSET" % v)
         else:
             warning("Attempt to UNSET nonexistent variable '%s'" % v)
 
@@ -720,13 +783,13 @@ class Macros:
 
         if cmd == 'ON':
             self.expandMode = 1
-            if gbl.debug:
-                print("Variable expansion ON")
+            if MMA.debug.debug:
+                dPrint("Variable expansion ON")
 
         elif cmd == 'OFF':
             self.expandMode = 0
-            if gbl.debug:
-                print("Variable expansion OFF")
+            if MMA.debug.debug:
+                dPrint("Variable expansion OFF")
 
         else:
             error("Use: Vexpand ON/Off")
@@ -760,8 +823,8 @@ class Macros:
 
         self.vars[v] = str(vl)
 
-        if gbl.debug:
-            print("Variable '%s' INC to %s" % (v, self.vars[v]))
+        if MMA.debug.debug:
+            dPrint("Variable '%s' INC to %s" % (v, self.vars[v]))
 
     def vardec(self, ln):
         """ Decrement a varaiable. """
@@ -791,8 +854,8 @@ class Macros:
 
         self.vars[v] = str(vl)
 
-        if gbl.debug:
-            print("Variable '%s' DEC to %s" % (v, self.vars[v]))
+        if MMA.debug.debug:
+            dPrint("Variable '%s' DEC to %s" % (v, self.vars[v]))
 
     def varIF(self, ln):
         """ Conditional variable if/then. """
