@@ -33,6 +33,8 @@ import MMA.alloc
 import MMA.volume
 import MMA.exits
 import MMA.regplug
+import MMA.debug
+import MMA.sync
 
 from . import gbl
 from MMA.common import *
@@ -75,37 +77,13 @@ def opts(l=None):
             setBarRange(a)
             gbl.barRange.append("ABS")
 
-        elif o == '-d':
-            gbl.debug = gbl.Ldebug = 1
-
-        elif o == '-o':
-            gbl.showFilenames = gbl.LshowFilenames = 1
-
-        elif o == '-p':
-            gbl.pshow = gbl.Lpshow = 1
-
-        elif o == '-s':
-            gbl.seqshow = gbl.Lseqshow = 1
-
+        elif o in ('-d', '-o', '-p', '-s', '-r', '-w', '-n', '-e', '-c'):
+            MMA.debug.cmdLineDebug(o[-1])
+        
         elif o == '-S':
             ln = a.split('=', 1)
             macros.setvar(ln)
 
-        elif o == '-r':
-            gbl.showrun = gbl.Lshowrun = 1
-
-        elif o == '-w':
-            gbl.noWarn = gbl.LnoWarn = 1
-
-        elif o == '-n':
-            gbl.noOutput = gbl.LnoOutput = 1
-
-        elif o == '-e':
-            gbl.showExpand = gbl.LshowExpand = 1
-
-        elif o == '-c':
-            gbl.chshow = gbl.Lchshow = 1
-
         elif o == '-L':
             gbl.printProcessed = True
 
@@ -186,10 +164,10 @@ def opts(l=None):
                 usage()
 
         elif o == '-0':
-            gbl.synctick = 1
+            MMA.sync.synchronize(['START'])
 
         elif o == '-1':
-            gbl.endsync = 1
+            MMA.sync.synchronize(['END'])
 
         elif o == '-P':
             gbl.playFile = 1
@@ -282,7 +260,7 @@ def opts(l=None):
     if gbl.infile == '-':
         gbl.infile = 1
 
-        if not gbl.outfile:
+        if not gbl.outfile and not(MMA.debug.noOutput):
             error("Input from STDIN specified. Use -f to set an output filename.")
 
 
@@ -291,7 +269,7 @@ def usage(msg=''):
 
     txt = [
         "MMA - Musical Midi Accompaniment",
-        "  Copyright 2003-14, Bob van der Poel. Version %s" % gbl.version,
+        "  Copyright 2003-19, Bob van der Poel. Version %s" % gbl.version,
         "  Distributed under the terms of the GNU Public License.",
         "  Usage: mma [opts ...] INFILE [opts ...]",
         "",
